
// Blockbuster
// Avisynth filter for eliminating DCT blocks
//
// By Ross Thomas <ross@grinfinity.com>
//
// There is no copyright on this code, and there are no conditions
// on its distribution or use. Do with it what you will.

#include "grand.h"

class NoiseBuffer
{
public:
	NoiseBuffer(int bytes, int seed, double mean, double variance, int block_size);
	virtual ~NoiseBuffer();

	char Next();

private:
	char* start, *end, *cur;
	int offset, block_bytes;
};

NoiseBuffer::NoiseBuffer(int bytes, int seed, double mean, double variance, int block_size)
{
	cur = start = new char[bytes];
	end = start + bytes;
	offset = 0;
	block_bytes = block_size * block_size;

	GaussianRandom r(seed);

	for(int i = 0; i < bytes; ++i)
	{
		double d = r.rand(mean, variance);

		if(d < 0.0) d -= 0.5;
		else d += 0.5;

		if(d < -128.0) *cur++ = -128;
		else if(d > 127.0) *cur++ = 127;
		else *cur++ = char(d);
	}

	cur = start;
}

NoiseBuffer::~NoiseBuffer()
{
	delete [] start;
}

inline char NoiseBuffer::Next()
{
	if(cur == end)
	{
		if(offset == block_bytes) offset = 0;
		cur = start + offset++;
	}
	return *cur++;
}
