
// Blockbuster
// Avisynth filter for eliminating DCT blocks
//
// By Ross Thomas <ross@grinfinity.com>
//
// There is no copyright on this code, and there are no conditions
// on its distribution or use. Do with it what you will.

// This method applies Gaussian noise to the block.

#include "Processor.h"
#include "NoiseBuffer.h"

class NoiseProcessor : public Processor
{
public:
	NoiseProcessor(int block_size, int detail_min,
		int detail_max, int luma_offset, int luma_threshold,
		double mean, double variance, int cache, int seed,
		IScriptEnvironment* env);

protected:
	virtual void ProcessBlock_YUY2(unsigned char* p, int pitch, int row_size,
		int height, int block_x, int block_y, IScriptEnvironment* env);

#ifdef AVISYNTH_25
	virtual void ProcessBlock_YV12(unsigned char* p, int pitch, int row_size,
		int height, int block_x, int block_y, IScriptEnvironment* env);
#endif

private:
	NoiseBuffer buf;
};

NoiseProcessor::NoiseProcessor(int block_size, int detail_min,
							   int detail_max, int luma_offset,
							   int luma_threshold, double mean,
							   double variance, int cache, int seed,
							   IScriptEnvironment* env)
	: Processor(block_size, detail_min, detail_max,
				luma_offset, luma_threshold, env),
	  buf(cache, seed, mean, variance, block_size)
{
	if(cache < 1)
		env->ThrowError("Blockbuster: Cache must be >= 1");
	if(seed < 0)
		env->ThrowError("Blockbuster: Seed must be >= 0");
}

void NoiseProcessor::ProcessBlock_YUY2(unsigned char* p, int pitch, int row_size,
									   int height, int /* block_x */, int /* block_y */,
									   IScriptEnvironment* /* env */)
{
	int skip = pitch - row_size;

	for(int y = 0; y < height; ++y)
	{
		int x = row_size >> 1;
		while(x-- != 0)
		{
			int n = *p + buf.Next();
			*p++ = unsigned char(MINMAX(n, 0, 255));
			++p;
		}
		p += skip;
	}
}

#ifdef AVISYNTH_25
void NoiseProcessor::ProcessBlock_YV12(unsigned char* p, int pitch, int row_size,
									   int height, int /* block_x */, int /* block_y */,
									   IScriptEnvironment* /* env */)
{
	int skip = pitch - row_size;

	for(int y = 0; y < height; ++y)
	{
		int x = 0;
		while(x++ < row_size)
		{
			int n = *p + buf.Next();
			*p++ = unsigned char(MINMAX(n, 0, 255));
		}
		p += skip;
	}
}
#endif
