
// Blockbuster
// Avisynth filter for eliminating DCT blocks
//
// By Ross Thomas <ross@grinfinity.com>
//
// There is no copyright on this code, and there are no conditions
// on its distribution or use. Do with it what you will.

// This method outlines the block to aid visualization of the
// effects of the various block-related parameters.

#include "Processor.h"

class ShowProcessor : public Processor
{
public:
	ShowProcessor(int block_size, int detail_min,
		int detail_max, int luma_offset, int luma_threshold,
		IScriptEnvironment* env);

protected:
	virtual void ProcessBlock_YUY2(unsigned char* p, int pitch, int row_size,
		int height, int block_x, int block_y, IScriptEnvironment* env);

#ifdef AVISYNTH_25
	virtual void ProcessBlock_YV12(unsigned char* p, int pitch, int row_size,
		int height, int block_x, int block_y, IScriptEnvironment* env);
#endif
};

ShowProcessor::ShowProcessor(int block_size, int detail_min,
							 int detail_max, int luma_offset,
							 int luma_threshold, IScriptEnvironment* env)
	: Processor(block_size, detail_min, detail_max,
				luma_offset, luma_threshold, env)
{
}

void ShowProcessor::ProcessBlock_YUY2(unsigned char* p, int pitch, int row_size,
									  int height, int /* block_x */, int /* block_y */,
									  IScriptEnvironment* /* env */)
{
	for(int x = 0; x < row_size; x += 2)
		p[x] = ~p[x];

	p += pitch;

	for(int y = 1; y < height - 1; ++y)
	{
		p[0] = ~p[0];
		p[row_size - 2] = ~p[row_size - 2];
		p += pitch;
	}

	for(int x = 0; x < row_size; x += 2)
		p[x] = ~p[x];
}

#ifdef AVISYNTH_25
void ShowProcessor::ProcessBlock_YV12(unsigned char* p, int pitch, int row_size,
									  int height, int /* block_x */, int /* block_y */,
									  IScriptEnvironment* /* env */)
{
	for(int x = 0; x < row_size; ++x)
		p[x] = ~p[x];

	p += pitch;

	for(int y = 1; y < height - 1; ++y)
	{
		p[0] = ~p[0];
		p[row_size - 1] = ~p[row_size - 1];
		p += pitch;
	}

	for(int x = 0; x < row_size; ++x)
		p[x] = ~p[x];
}
#endif
