
// Blockbuster
// Avisynth filter for eliminating DCT blocks
//
// By Ross Thomas <ross@grinfinity.com>
//
// There is no copyright on this code, and there are no conditions
// on its distribution or use. Do with it what you will.

#pragma once

#include "urand.h"
#include <math.h>

class GaussianRandom : public Random
{
public:
	GaussianRandom(int seed);
	virtual double rand();
	double rand(double mean, double variance);

private:
	UniformRandom uni;
	bool iset;
	double gset;
};

GaussianRandom::GaussianRandom(int seed)
	: uni(seed)
{
	iset = false;
}

double GaussianRandom::rand()
{
	double fac, rsq, v1, v2;

	if(iset)
	{
		iset = false;
		return gset;
	}

	do
	{
		v1 = 2.0 * uni.rand() - 1.0;
		v2 = 2.0 * uni.rand() - 1.0;
		rsq = v1 * v1 + v2 * v2;
	} while(rsq >= 1.0 || 0.0 == rsq);

	fac = sqrt(-2.0 * log(rsq) / rsq);

	gset = v1 * fac;
	iset = true;

	return v2 * fac;
}

inline double GaussianRandom::rand(double mean, double variance)
{
	if(variance == 0.0) return mean;
	return rand() * sqrt(variance) + mean;
}
