
// Blockbuster
// Avisynth filter for eliminating DCT blocks
//
// By Ross Thomas <ross@grinfinity.com>
//
// There is no copyright on this code, and there are no conditions
// on its distribution or use. Do with it what you will.

#pragma once

#include "rand.h"
#include <time.h>

#define IA 16807
#define IM 2147483647
#define AM (1.0f / IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1 + (IM - 1) / NTAB)
#define EPS 1.2e-7f
#define RNMX (1.0f - EPS)

class UniformRandom : public Random
{
public:
	UniformRandom(int seed);
	virtual double rand();

private:
	long idum, iy, iv[NTAB];
};

UniformRandom::UniformRandom(int seed)
{
	assert(seed >= 0);
	if(0 == seed)
		idum = -(long)time(NULL);
	else
		idum = -seed;
	iy = 0;
}

double UniformRandom::rand()
{
	int j;
	long k;
	double temp;

	if(idum <= 0 || !iy)
	{
		if(-idum < 1)
			idum = 1;
		else
			idum = -idum;
		for(j = NTAB + 7; j >= 0; --j)
		{
			k = idum / IQ;
			idum = IA * (idum - k * IQ) - IR * k;
			if(idum < 0)
				idum += IM;
			if(j < NTAB)
				iv[j] = idum;
		}
		iy = iv[0];
	}
	k = idum / IQ;
	idum = IA * (idum - k * IQ) - IR * k;
	if(idum < 0)
		idum += IM;
	j = iy / NDIV;
	iy = iv[j];
	iv[j] = idum;
	temp = AM * iy;
	return temp > RNMX ? RNMX : temp;
}
